(ns wanijo.instance.view
  (:require [hiccup
             [form :as hform]
             [core :refer [h]]]
            [ring.util.anti-forgery :refer [anti-forgery-field]]
            [markdown.core :as md]
            [formulare.core :as form]
            [wanijo.instance.domain :as domain]
            [wanijo.framework
             [view :as view]
             [routing :refer [path]]
             [time :refer [prettify-dt]]]))

(defn list! [schema instances new-form req]
  (view/layout!
   :request req
   :content
   [[:h1 "All Instances of schema "
     [:span.schema-title__name (h (:name schema))]]
    [:table
     [:thead
      [:tr
       [:th "Name"]
       [:th "Updated"]
       [:th "Created"]]]
     [:tbody
      (for [instance instances]
        [:tr
         [:td
          [:a {:href (path :instance-show instance)}
           (h (:name instance))]]
         [:td (prettify-dt (:updated_at instance))]
         [:td (prettify-dt (:created_at instance))]])]]
    [:h1 "New Instance"]
    (hform/form-to [:post (path :instance-new)]
                   (form/render-widgets new-form {} req)
                   (hform/hidden-field "schema-uuid"
                                       (:uuid schema))
                   (hform/submit-button "Create!"))]))

(defn show! [instance req]
  (view/layout!
   :request req
   :content
   [[:h1
     (h (-> instance :schema :name))
     "&nbsp;"
     [:small (h (:name instance))]]
    [:p
     [:small [:a {:href (path :instance-edit-form instance)}
              "Edit Instance"]]]
    [:h2 "Properties"]
    (for [prop (:properties instance)
          :let [attr (:attribute prop)
                type (:type attr)
                render-fn (case type
                            "date" #(str (prettify-dt %))
                            "markdown" md/md-to-html-string
                            :else #(str "<p>" % "</p>"))]]
      (list [:h3 (h (:name attr))]
            [:div {:class (str "instance-content "
                               "attr-type-" type)}
             (render-fn (:value prop))]))]))

(defn edit! [instance form form-data req]
  (view/layout!
   :request req
   :content
   [[:h1
     (h (-> instance :schema :name))
     "&nbsp;"
     [:small (h (:name instance))]]
    (hform/form-to [:post (path :instance-edit instance)]
                   (form/render-widgets form form-data req)
                   (hform/submit-button "Edit!"))
    (hform/form-to [:delete ""]
                   (anti-forgery-field)
                   (view/delete-btn))]))
